;; Generated package description from realgud-jdb.el
(define-package "realgud-jdb" "1.0.0" "Realgud front-end to older nodejs \"node debug\"" '((realgud "1.4.5") (load-relative "1.2") (cl-lib "0.5") (emacs "25")) :url "http://github.com/realgud/realgud-jdb" :keywords nil)
